<?php

class ebooksController extends Controller {
    
    private $model;
    private $pagination_limit = 10;
    private $pagination_buttons;

    function __construct()
    {
        $this->pagination_buttons = array(__('Previous'), __('Next'));
        session::check('user_id', SITE_URL.'default/');
        if(session::get('user_role') != 'A')
        {            
            session::clean_session(SITE_URL.'login/');
            die();
        }


        $this->data['footer'] = $this->model('settings')->get_settings()[0]->institute_name;
        $this->data['footer_address'] = $this->model('settings')->get_settings()[0]->address;
        $this->data['logo'] = $this->model('settings')->get_settings()[0]->logo;

        $this->data['categories'] =  $this->model('books')->get_categories();
    }

    function actionMain($page = 1)
    {
        $modal = $this->model('books');

        if(isset($_GET['export']))
        {
            $modal->process_ebook_export();
        }
        else
        {
            $this->prg('prg_data',SITE_URL.'ebooks/');
            $filter = array();
            $limit = 'limit 0,'.$this->pagination_limit;
            $this->data['pagination_link'] = '';

            if(isset($_POST['pagination']))
            {
                $page = (int)$_POST['filter'];
            }
            if($page > 1)
            {
                $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
            }

            if(isset($_POST['filter']))
            {

                if(isset($_POST['author_filter']) && $_POST['author_filter'] != '')
                {
                    $filter['author'] = $_POST['author_filter'];
                }
                if(isset($_POST['name_filter']) && $_POST['name_filter'] != '')
                {
                    $filter['name'] = $_POST['name_filter'];
                }
                if(isset($_POST['isbn_filter']) && $_POST['isbn_filter'] != '')
                {
                    $filter['isbn'] = $_POST['isbn_filter'];
                }
                if(isset($_POST['category_filter']) && $_POST['category_filter'] != '')
                {
                    $filter['category'] = $_POST['category_filter'];
                }
                $this->data['filter'] = $filter;
            }

            $this->data['books'] = $modal->get_ebooks($filter, $limit);
            $count = count( $modal->get_ebooks($filter));

            $this->data['modal'] = $modal;
            $this->error['error_code'] = $modal->flag;
            $this->error['message'] = $modal->message;

            if($page > 1)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
            }
            if($count > count($this->data['books']) * $page && count($this->data['books']) > 0)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
            }


            $this->data['category'] = $modal->get_categories();
            $this->data['modal'] = $modal;

            $this->data['page_title'] = __('E-Books | Library Management System');

            $this->view('header');
            $this->view('view_ebooks','books');
            $this->view('footer');
        }
    }

    
    function actionAddEbook()
    {
        $modal = $this->model('books');

            if(isset($_POST['submit']))
            {
                $modal->process_ebook();

                $this->error['message'] = $modal->message;
                $this->error['error_code'] = $modal->flag;
            }

            $this->data['categories'] = $modal->get_categories();
            
            $this->data['page_title'] = __('Add E-Book | Library Management System');

            $this->view('header');
            $this->view('addebook','books');
            $this->view('footer');
    }


    function actionDeletebook()
    {
         if(Helpers::isAjax())
        {
            $modal = $this->model('books');

            if(isset($_GET['id']))
            {
                    $modal->delete_ebook($_GET['id']);
            }

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }

    function actionUpdatebook()
    {
        if(Helpers::isAjax())
        {
            $modal = $this->model('books');

            if(isset($_POST['type']) && $_POST['type'] == 'single')
                $modal->update_ebook('single');
            else
                $modal->update_ebook();

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }

    function actionEdit()
    {
        $modal = $this->model('books');
        if(isset($_GET['id']) && is_numeric($_GET['id']))
        {
            $book = $modal->check_ebook($_GET['id']);
            if (!empty($book) && $book != false) 
            {
                if(isset($_POST['submit']))
                {
                    if($_POST['security_token'] !== $_SESSION['token']) 
                    {
                        $this->error['message'] = __('Token Mismatch');
                        $this->error['error_code'] = ERROR;
                    }
                    else
                    {
                        $modal->update_ebook('all');

                        $this->error['message'] = $modal->message;
                        $this->error['error_code'] = $modal->flag;
                    }
                   
                }
        /**
         *
         *
         *
         * Generate security token to handle double form submissions
         *
         *
         *
         **/
                $this->data['token'] = md5(uniqid(rand(), TRUE).time());
                $_SESSION['token'] = $this->data['token'];

                // update form after submit
                $book = $modal->check_ebook($_GET['id']);

                $this->data['categories'] = $modal->get_categories();

                $this->data['title'] = $book->name;
                $this->data['description'] = $book->description;
                $this->data['language'] = $book->language;
                $this->data['category'] = $modal->get_categories($book->category);
                $this->data['publisher'] = $book->publisher;
                $this->data['photo'] = $book->photo;
                $this->data['publishedDate'] = $book->published_year;
                $this->data['authors'] = $book->author;
                $this->data['e_file'] = $book->download_file;

                $this->view('header');
                $this->view('edit_ebook_details','books');
                $this->view('footer');
            }
            else
            {
                header('HTTP/1.1 405'.__('Requested Method Not Allowed'));
                echo __('Invalid access! Go').'<a href="'.SITE_URL.'books/viewbooks">Home</a>';
                die();
            }
        }
        else
        {
            header('HTTP/1.1 405'.__('Requested Method Not Allowed'));
            echo __('Invalid access! Go').'<a href="'.SITE_URL.'books/viewbooks">Home</a>';
            die();
        }
    }
}
?>