<?php

class usersController extends Controller {
    
    private $model;
    private $pagination_limit = 10;
    private $pagination_buttons;


    function __construct() {
        $this->pagination_buttons = array(__('Prevoius'), __('Next'));
        session::check('user_id', SITE_URL.'login/');
        if(session::get('user_role') != 'A')
        {            
            session::clean_session(SITE_URL.'login/');
            die();
        }

        $this->data['footer'] = $this->model('settings')->get_settings()[0]->institute_name;
        $this->data['footer_address'] = $this->model('settings')->get_settings()[0]->address;
        $this->data['logo'] = $this->model('settings')->get_settings()[0]->logo;
    }

    
    function actionMain()
    {
        $modal = $this->model('users');
        $this->data['total_users'] = count($modal->get_users());
        $this->data['total_students'] = count($modal->get_students());
        $this->data['total_teachers'] = count($modal->get_teachers());

        $this->data['class'] = $this->model('users')->get_classes();

        $this->data['page_title'] = __('Dashboard users | Library Management System');

        $this->view('header');
        $this->view('dashboard','users');
        $this->view('footer');
    }
    function actionAddStudent()
    {
        $modal = $this->model('users');
        if(isset($_POST['submit'])) {
            $result = $modal->process_student();
            $this->error['error_code'] = $modal->flag;
            $this->error['message'] = $modal->message;
        }

        $this->data['class'] = $modal->get_classes();


        $this->data['page_title'] = __('Add Student | Library Management System');

        $this->view('header');
        $this->view('addstudent','users');
        $this->view('footer');
    }
    
    function actionAddTeacher()
    {
        $modal = $this->model('users');
        if(isset($_POST['submit'])) {
            $result = $modal->process_teacher();
            $this->error['error_code'] = $modal->flag;
            $this->error['message'] = $modal->message;
        }

        $this->data['designation'] = $modal->get_designations();

        $this->data['page_title'] = __('Add Teacher | Library Management System');

        $this->view('header');
        $this->view('addteacher','users');
        $this->view('footer');
    }
    
    function actionTeachers($page = 1)
    {
        $modal = $this->model('users');

        if(isset($_GET['export']))
        {
            $modal->process_export_teachers();
        }
        else
        {
            $this->prg('prg_data',SITE_URL.'users/teachers/');

            $filter = array();
            $limit = 'limit 0,'.$this->pagination_limit;
            $this->data['pagination_link'] = '';

            if(isset($_POST['pagination']))
            {
                $page = (int)$_POST['filter'];
            }

            if($page > 1)
            {
                $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
            }

            if(isset($_POST['filter']))
            {

                if(isset($_POST['designation_filter']) && $_POST['designation_filter'] != '')
                {
                    $filter['designation'] = $_POST['designation_filter'];
                }
                if(isset($_POST['member_id_filter']) && $_POST['member_id_filter'] != '')
                {
                    $filter['member_id'] = $_POST['member_id_filter'];
                }
                if(isset($_POST['name_filter']) && $_POST['name_filter'] != '')
                {
                    $filter['name'] = $_POST['name_filter'];
                }
                if(isset($_POST['email_filter']) && $_POST['email_filter'] != '')
                {
                    $filter['email'] = $_POST['email_filter'];
                }
                $this->data['filter'] = $filter;
            }
            $this->data['teachers'] = $modal->get_teachers($filter,$limit);
            $count = count($modal->get_teachers($filter));

            if($page > 1)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
            }
            if($count > count($this->data['teachers']) * $page && count($this->data['teachers']) > 0)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
            }

            $this->data['designation'] = $modal->get_designations();

            $this->data['page_title'] = __('View Teachers | Library Management System');

            $this->view('header');
            $this->view('view_teacher','users');
            $this->view('footer');
        }
    }
    
    function actionStudents($page = 1)
    {
        $modal = $this->model('users');
        if(isset($_GET['export']))
        {
            $modal->process_export_students();
        }
        else {
            $this->prg('prg_data',SITE_URL.'users/students/');
            $this->data['class'] = $this->model('classes')->get_class();
            
            $filter = array();
            $limit = 'limit 0,'.$this->pagination_limit;
            $this->data['pagination_link'] = '';

            if(isset($_POST['pagination']))
            {
                $page = (int)$_POST['filter'];
            }
            
            if($page > 1)
            {
                $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
            }


            if(isset($_POST['filter']))
            {

                if(isset($_POST['class_filter']) && $_POST['class_filter'] != '')
                {
                    $filter['class'] = $_POST['class_filter'];
                }
                if(isset($_POST['reg_filter']) && $_POST['reg_filter'] != '')
                {
                    $filter['reg_no'] = $_POST['reg_filter'];
                }
                if(isset($_POST['roll_no_filter']) && $_POST['roll_no_filter'] != '')
                {
                    $filter['roll_no'] = $_POST['roll_no_filter'];
                }
                if(isset($_POST['name_filter']) && $_POST['name_filter'] != '')
                {
                    $filter['name'] = $_POST['name_filter'];
                }
                $this->data['filter'] = $filter;
            }

            $this->data['students'] = $modal->get_students($filter,$limit);
            $this->data['modal'] = $modal;
            $count = count($modal->get_students($filter));

            if($page > 1)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
            }
            if($count > count($this->data['students']) * $page && count($this->data['students']) > 0)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
            }

            $this->data['page_title'] = __('View Students | Library Management System');

            $this->view('header');
            $this->view('view_student','users');
            $this->view('footer');
        }
    }

    function actionDeleteStudent()
    {
         if(Helpers::isAjax())
        {
            $modal = $this->model('users');

            if(isset($_GET['id']))
            {
                    $modal->delete_student($_GET['id']);
            }

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }

    function actionDeleteTeacher()
    {
         if(Helpers::isAjax())
        {
            $modal = $this->model('users');

            if(isset($_GET['id']))
            {
                    $modal->delete_teacher($_GET['id']);
            }

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }

    function actionUpdateStudent()
    {
        if(Helpers::isAjax())
        {
            $modal = $this->model('users');

            $modal->update_student();

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }

    function actionUpdateTeacher()
    {
        if(Helpers::isAjax())
        {
            $modal = $this->model('users');

            $modal->update_teacher();

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }

    function actionEditStudent($regenerate = false)
    {
        $modal = $this->model('users');
        if($regenerate == true) {

            $user = $modal->check_student($_GET['id']);
            if (count($user) > 0 && $user != false) 
            {
                if($_GET['token'] !== $_SESSION['token']) 
                {
                    $this->error['message'] = __('Token Mismatch');
                    $this->error['error_code'] = ERROR;
                }

                else {
                    $modal->regenerate_password($_GET['id']);

                    $this->error['message'] = $modal->message;
                    $this->error['error_code'] = $modal->flag;
                }
            }
        }
        if(isset($_GET['id']) && is_numeric($_GET['id']))
        {
            $user = $modal->check_student($_GET['id']);
            if (count($user) > 0 && $user != false) 
            {
                if(isset($_POST['submit']))
                {
                    if($_POST['security_token'] !== $_SESSION['token']) 
                    {
                        $this->error['message'] = __('Token Mismatch');
                        $this->error['error_code'] = ERROR;
                    }
                    else
                    {
                        $modal->update_student('all');

                        $this->error['message'] = $modal->message;
                        $this->error['error_code'] = $modal->flag;
                    }
                   
                }
        /**
         *
         *
         *
         * Generate security token to handle double form submissions
         *
         *
         *
         **/
                $this->data['token'] = md5(uniqid(rand(), TRUE).time());
                $_SESSION['token'] = $this->data['token'];

                // update form after submit
                $student = $modal->check_student($_GET['id']);

                $this->data['class'] = $modal->get_classes();

                $this->data['name'] = $student->name;
                $this->data['email'] = $student->email;
                $this->data['username'] = $student->username;
                $this->data['phone'] = $student->phone;
                $this->data['address'] = $student->address;
                $this->data['reg_no'] = $student->registration_no;
                $this->data['father_name'] = $student->fathers_name;
                $this->data['class_name'] = $student->class;
                $this->data['roll'] = $student->roll_no;

                $this->view('header');
                $this->view('edit_student','users');
                $this->view('footer');
            }
            else
            {
                header('HTTP/1.1 405'.__('Requested Method Not Allowed'));
                echo __('Invalid access! Go ').'<a href="'.SITE_URL.'users/students">Home</a>';
                die();
            }
        }
        else
        {
            header('HTTP/1.1 405'.__('Requested Method Not Allowed'));
                echo __('Invalid access! Go ').'<a href="'.SITE_URL.'users/students">Home</a>';
            die();
        }
    }

    function actionEditTeacher($regenerate = false)
    {
        $modal = $this->model('users');
        if($regenerate == true) {

            $user = $modal->check_teacher($_GET['id']);
            if (count($user) > 0 && $user != false) 
            {
                if($_GET['token'] !== $_SESSION['token']) 
                {
                    $this->error['message'] = __('Token Mismatch');
                    $this->error['error_code'] = ERROR;
                }

                else {
                    $modal->regenerate_password($_GET['id']);

                    $this->error['message'] = $modal->message;
                    $this->error['error_code'] = $modal->flag;
                }
            }
        }

        if(isset($_GET['id']) && is_numeric($_GET['id']))
        {
            $user = $modal->check_teacher($_GET['id']);
            if (count($user) > 0 && $user != false) 
            {
                if(isset($_POST['submit']))
                {
                    if($_POST['security_token'] !== $_SESSION['token']) 
                    {
                        $this->error['message'] = __('Token Mismatch');
                        $this->error['error_code'] = ERROR;
                    }
                    else
                    {
                        $modal->update_teacher('all');

                        $this->error['message'] = $modal->message;
                        $this->error['error_code'] = $modal->flag;
                    }
                   
                }
        /**
         *
         *
         *
         * Generate security token to handle double form submissions
         *
         *
         *
         **/
                $this->data['token'] = md5(uniqid(rand(), TRUE).time());
                $_SESSION['token'] = $this->data['token'];

                // update form after submit
                $student = $modal->check_teacher($_GET['id']);

                $this->data['designation'] = $modal->get_designations();

                $this->data['member_id'] = $student->member_id;
                $this->data['faculty_name'] = $student->name;
                $this->data['email'] = $student->email;
                $this->data['username'] = $student->username;
                $this->data['phone'] = $student->phone;
                $this->data['address'] = $student->address;
                $this->data['designation_name'] = $student->designation;

                $this->view('header');
                $this->view('edit_teacher','users');
                $this->view('footer');
            }
            else
            {
                header('HTTP/1.1 405 '.__('Requested Method Not Allowed'));
                echo __('Invalid access! Go ').'<a href="'.SITE_URL.'users/teachers">Home</a>';
                die();
            }
        }
        else
        {
            header('HTTP/1.1 405 '.__('Requested Method Not Allowed'));
                echo __('Invalid access! Go ').'<a href="'.SITE_URL.'users/teachers">Home</a>';
            die();
        }
    }


    function actionImportTeachers()
    {
        if(Helpers::isAjax())
        {
            $modal = $this->model('users');

            $modal->process_import_teachers();

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }


    function actionImportStudents()
    {
        if(Helpers::isAjax())
        {
            $modal = $this->model('users');

            $modal->process_import_students();

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }



}
?>