<?php
/**
 * File contains necessary information that defines the bootstrap of your application
 * Source code pattern must not be modified
 * Files to be used with the comments.
 * @Author : Operce Technologies
 * @Developer : Manhar Sharma
 * @Year : 2016
 *
 *
 **/


/**
 * class for login
 * pattern must not be modified
 * Files to be used with the comments.
 *
 *
 */

defined('_DONUT') or die('Error. Invalid Access');

class issue extends Model {
    public $page_title;
    public $message;
    public $flag;
    public $values;
    private $crud;

    function __construct() {
        $this->crud = new crud('lms_issue');
        $this->check_fine(true);
    }

    function process_issue() {
        $this->values = $_POST;
        if(!$this->check_form($this->values, ['book_id'=>'Book ID', 'user_id' => 'User ID', 'days' => 'Days'], '{book_id},{days}')) {
            $this->flag = ERROR;
            return false;
        }

        $this->add_issue();
    }

    private function add_issue() {
        
        $days = $this->check_days();
        $user = $this->check_user();
        $book = $this->check_book();

        if($book == false || $user == false || $days == false)
        {
            return false;
        }

        $prev = $this->check_prev($user[0]->user_id, $book[0]->book_id, $book[0]->quantity);

        if($prev == false)
        {
            return false;
        }

        $dt = new DateTime();
        $date = $dt->format('Y-m-d H:i:s');
        $data = $this->crud->insert( array('user_id'=>$user[0]->user_id, 'issued_days'=>$this->values['days'], 'book_id'=>$book[0]->book_id, 'fine' => '0', 'status' => 'issue', 'date' => $date) );
        
        $this->crud->changeTable('lms_requested');
        $check = $this->crud->get_by(array('user_id' => $user[0]->user_id, 'book_id' => $book[0]->book_id));
        
        if(count($check) > 0)
            $this->crud->delete(array('user_id' => $user[0]->user_id, 'book_id' => $book[0]->book_id));

        if($data > 0)
        {
            $this->flag = SUCCESS;
            $this->message = __(' Book issued to ther user');
            return true;
        }
        else
        {
            $this->flag = ERROR;
            $this->message = __("Error adding Book to the issued list");
            return true;
        }
        
    }
    
    function process_requested() {
        $this->values = $_GET;
        if(!$this->check_form($this->values, ['book_id'=>'Book ID'])) {
            $this->flag = ERROR;
            return false;
        }

        $this->add_requested();
    }

    private function add_requested() {
       
       $book = $this->check_book();

        if($book == false)
        {
            $this->flag = ERROR;
            $this->message = __(' Book request error');
            return false;
        }

        $prev = $this->check_prev(session::get('user_id'), $book[0]->book_id,  $book[0]->quantity);
        $prev_req = $this->check_prev_req(session::get('user_id'), $book[0]->book_id);

        if($prev == false || $prev_req == false)
        {
            $this->flag = ERROR;
            $this->message = __(' Book request error. Either book is already requested or it is not availbale. Please contact administrator');
            return false;
        }

        $dt = new DateTime();
        $date = $dt->format('Y-m-d H:i:s');

        $this->crud->changeTable('lms_requested');
        $data = $this->crud->insert( array('user_id'=>session::get('user_id'), 'book_id'=>$book[0]->book_id, 'date' => $date) );
        
        if($data > 0)
        {
            $this->flag = SUCCESS;
            $this->message = __(' Book request completed');
            return true;
        }
        else
        {
            $this->flag = ERROR;
            $this->message = __("Error requesting book");
            return true;
        }
        
    }


    function cancel_requested() {
       
        $this->values = $_GET;
        if (!is_numeric($_GET['book_id']) || !is_numeric($_GET['user_id']) || (empty($_GET['book_id']) || empty($_GET['user_id']))) 
        {
            $this->flag = ERROR;
            $this->message = __("Invalid access");
            return true;
        }

        if(!$this->check_form($this->values, ['book_id'=>'Book ID', 'user_id' => 'user id'])) {
            $this->flag = ERROR;
            return false;
        }
        
        $this->crud->changeTable('lms_requested');
        $check = $this->crud->get_by(array('user_id' => $this->values['user_id'], 'book_id' => $this->values['book_id']));

        if(count($check) < 0)
        {
            $this->flag = ERROR;
            $this->message = __("Invalid access");
            return true;
        }
        
        $data = $this->crud->delete( array('user_id' => $this->values['user_id'], 'book_id' => $this->values['book_id']) );
        
        if($data > 0)
        {
            $this->flag = SUCCESS;
            $this->message = __(' Request cancelled');
            return true;
        }
        else
        {
            $this->flag = ERROR;
            $this->message = __("Error Cancelling book");
            return true;
        }
        
    }

    function check_prev_req($user_id, $book_id)
    {
        $query = "select * from lms_requested where book_id = :book_id AND user_id = :user_id";
        $book = $this->crud->exec_query($query, array(':book_id' => $book_id, ':user_id' => $user_id))->fetchAll(PDO::FETCH_CLASS);
        
        if(count($book) > 0)
        {
            $this->flag = ERROR;
            $this->message = __("Book already issued to user");
            return false;
        }

        return true;
    }


    function check_user()
    {
        $user = array();
        if($this->values['type'] == 'student')
        {
            $query = 'select * from lms_students where user_id = :book_id OR registration_no = :book_id OR roll_no = :book_id';
            $user = $this->crud->exec_query($query, array(':book_id' => $this->values['user_id']))->fetchAll(PDO::FETCH_CLASS);
        }
        elseif($this->values['type'] == 'teacher')
        {
            $query = 'select * from lms_staff where user_id = :book_id OR member_id = :book_id';
            $user = $this->crud->exec_query($query, array(':book_id' => $this->values['user_id']))->fetchAll(PDO::FETCH_CLASS);
        }
        else
        {
            $this->flag = ERROR;
            $this->message = __("Invalid Access. Please start again");
            return false;
        }
        if(count($user) <= 0)
        {
            $this->flag = ERROR;
            $this->message = __("User ID not valid");
            return false;
        }

        return $user;
    }


    function check_book()
    {
        $query = 'select * from lms_books where book_id = :book_id OR barcode_id = :book_id OR isbn = :book_id';
        $book = $this->crud->exec_query($query, array(':book_id' => $this->values['book_id']))->fetchAll(PDO::FETCH_CLASS);
        
        if(count($book) <= 0)
        {
            $this->flag = ERROR;
            $this->message = __("Invalid Book Details");
            return false;
        }
        return $book;
    }

    function check_prev($user_id, $book_id, $book_count)
    {
        $query = "select * from lms_issue where book_id = :book_id AND user_id = :user_id AND status = 'issue'";
        $book = $this->crud->exec_query($query, array(':book_id' => $book_id, ':user_id' => $user_id))->fetchAll(PDO::FETCH_CLASS);
        
        if(count($book) > 0)
        {
            $this->flag = ERROR;
            $this->message = __("Book already issued to user");
            return false;
        }
        else
        {
            $query = "select * from lms_issue where book_id = :book_id AND status = 'issue'";
            $count = $this->crud->exec_query($query, array(':book_id' => $book_id))->fetchAll(PDO::FETCH_CLASS);
            
            if(($book_count - count($count)) <= 0)
            {
                $this->flag = ERROR;
                $this->message = __("Book is out of stock. Cannot issue book.");
                return false;
            }
            return true;
        }
    }

    function check_days()
    {
        $query = 'select * from lms_settings';
        $days = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);

        if($this->values['days'] == '')
        {
            $this->values['days'] = $days[0]->max_days;
        }
        else if($days[0]->max_days < $this->values['days'] || $this->values['days'] < $days[0]->min_days)
        {            
            $this->flag = ERROR;
            $this->message = __("Invalid issue period");
            return false;
        }

        if($this->values['days'] <= 0)
        {
            $this->flag = ERROR;
            $this->message = __("Invalid issue period");
            return false;
        }

        return $days;
    }


    function get_user_id()
    {
        $this->values = $_POST;
        if(!$this->check_form($this->values, ['type' => 'User Type', 'user_id' => 'User ID'])) {
            $this->flag = ERROR;
            return false;
        }

        $user = $this->check_user();

        $args = array('user_id' => $user[0]->user_id, 'check' => sha1(AUTH_KEY.$user[0]->user_id));

        return $args;
    }


    function get_books()
    {
        if($_POST['type'] == 'student')
        {
            $query = "select lms_issue.date as issued_date, lms_issue.id as issue_id, lms_issue.*, lms_books.name as book_name, lms_books.*, lms_users.*, lms_students.* from lms_issue inner join lms_users on lms_issue.user_id = lms_users.user_id inner join lms_books on lms_books.book_id = lms_issue.book_id inner join lms_students on lms_students.user_id = lms_users.user_id where lms_issue.status = 'issue' AND (lms_issue.user_id = '".$_POST['user_id'] ."' OR lms_students.roll_no = '".$_POST['user_id']."' OR lms_students.registration_no = '".$_POST['user_id']."')";
        }

        else if($_POST['type'] == 'teacher')
            $query = "select lms_issue.date as issued_date, lms_issue.id as issue_id, lms_issue.*, lms_books.name as book_name, lms_books.*, lms_users.*, lms_staff.* from lms_issue inner join lms_users on lms_issue.user_id = lms_users.user_id inner join lms_books on lms_books.book_id = lms_issue.book_id inner join lms_staff on lms_staff.user_id = lms_users.user_id where lms_issue.status = 'issue' AND (lms_issue.user_id = '".$_POST['user_id'] ."' OR lms_staff.member_id = '".$_POST['user_id']."')";


        else
        {
            $this->flag = ERROR;
            $this->message = __("Invalid request");
            return false;
        }

        $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);

        if(count($data) <= 0)
        {

            $this->flag = WARNING;
            $this->message = __("No books issued till date");
        }
        
        return $data;
    }

    function check_fine($status = false)
    {
        if($status == true)
        {
            $query = "select lms_users.type, lms_issue.* from lms_issue inner join lms_users on lms_users.user_id = lms_issue.user_id where lms_issue.status = 'issue'" ;
            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            $fine = 0;
            $query = 'select fine, fine_to_teacher from lms_settings';
            $fine_amt = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS)[0]->fine;
            $fine_amt_teacher = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS)[0]->fine_to_teacher;

            if(count($data) > 0)
            {
                foreach ($data as $key => $value) {
                    $date = $value->date;
                    $date = new DateTime(date('Y/m/d', strtotime($date. ' + '.$value->issued_days.' days')));

                    $curr_date = new DateTime(date('Y/m/d'));
                    $interval = $curr_date->diff($date);

                    if($interval->days > 0 && $interval->invert == 1)
                    {
                        if($value->type == 'T') {
                             $fine = $fine_amt_teacher * $interval->days;
                        }
                        else {
                            $fine = $fine_amt * $interval->days;
                        }
                        $query = 'update lms_issue set fine = '.$fine.' where lms_issue.id = '.$value->id;
                        $data = $this->crud->exec_query($query);
                    }
                }
            }
        }
        else
        {
            $query = "select * from lms_issue where status = 'issue' and user_id = (select lms_students.user_id from lms_students where lms_students.user_id = ".$_POST['user_id']." or lms_students.roll_no = ".$_POST['user_id']." or lms_students.registration_no = ".$_POST['user_id'].")" ;
            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            $fine = 0;
            $query = 'select fine from lms_settings';
            $fine_amt = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS)[0]->fine;

            if(count($data) > 0)
            {
                foreach ($data as $key => $value) {
                    $date = $value->date;
                    $date = new DateTime(date('Y/m/d', strtotime($date. ' + '.$value->issued_days.' days')));

                    $curr_date = new DateTime(date('Y/m/d'));
                    $interval = $curr_date->diff($date);

                    if($interval->days > 0 && $interval->invert == 1)
                    {
                        $fine = $fine_amt * $interval->days;
                        $query = 'update lms_issue set fine = '.$fine.' where lms_issue.id = '.$value->id;
                        $data = $this->crud->exec_query($query);
                    }
                }
            }
        }
    }


    function process_return_book()
    {
        $this->values = array('id' => $_GET['id']);
        if(!$this->check_form($this->values, ['id' => 'Book'])) {
            $this->flag = ERROR;
            $this->message = __("Invalid request");
            return false;
        }

        $data = $this->crud->update( array('status' => 'return', 'return_date' => (new \DateTime())->format('Y-m-d H:i:s')), array( 'id' => $this->values['id'] ));

        if($data > 0)
        {
            $this->flag = SUCCESS;
            $this->message = __(' Updated Successfully');
            return true;
        }
        else
        {
            $this->flag = WARNING;
            $this->message = __("Nothing was updated");
            return true;
        }
    }

    function process_lost_book()
    {
        $this->values = array('id' => $_GET['id']);
        if(!$this->check_form($this->values, ['id' => 'Book'])) {
            $this->flag = ERROR;
            $this->message = __("Invalid request");
            return false;
        }

        $book_id = $this->crud->get_by( array( 'id' => $this->values['id'] ), '=', true );

        $this->crud->changeTable('lms_books');
        $book = $this->crud->get_by( array( 'book_id' => $book_id->book_id ), '=', true);

        if(count($book) <= 0)
        {
            $this->flag = ERROR;
            $this->message = __('Something went wront. Please contact administrator');
            return false;
        }

        if($book->quantity > 0)
        {
            $this->crud->update( array('quantity' => ((int)$book->quantity - 1) ), array( 'book_id' => $book_id->book_id ) );
        }

        $this->crud->changeTable('lms_issue');

        $data = $this->crud->update( array('status' => 'lost'), array( 'id' => $this->values['id'] ) );

        if($data > 0)
        {
            $this->flag = SUCCESS;
            $this->message = __(' Updated Successfully');
            return true;
        }
        else
        {
            $this->flag = WARNING;
            $this->message = __("Nothing was updated");
            return true;
        }
    }


    function get_issued($filter = array(), $limit = false)
    {
        if(count($filter) <= 0 && $limit == false)
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.email as email,lms_users.*,lms_issue.id as issue_id, lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $query = $query . " where lms_issue.status = 'issue'";
            

            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }

        else
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.email as email,lms_users.*,lms_issue.id as issue_id, lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            
            $query = ($flag == true) ? $query . " AND lms_issue.status = 'issue'" : $query . " where lms_issue.status = 'issue'";
            
            if($limit != false)
            {
                $query = $query.' '.$limit;
            }

            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }
    }

    function get_delayed($filter = array(), $limit = false)
    {
        if(count($filter) <= 0 && $limit == false)
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.email as email,lms_users.*,lms_issue.id as issue_id, lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            
            
            $query = $query . " where lms_issue.status = 'issue'";
            $query = $query . " AND DATE_ADD(lms_issue.date, INTERVAL lms_issue.issued_days DAY) < now()";


            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }

        else
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.email as email,lms_users.*,lms_issue.id as issue_id, lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            
            $query = ($flag == true) ? $query . " AND lms_issue.status = 'issue'" : $query . " where lms_issue.status = 'issue'";
            $query = $query . " AND DATE_ADD(lms_issue.date, INTERVAL lms_issue.issued_days DAY) < now()";

            if($limit != false)
            {
                $query = $query.' '.$limit;
            }

            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }
    }

    function get_lost($filter = array(), $limit = false)
    {
        if(count($filter) <= 0 && $limit == false)
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.*,lms_issue.id as issue_id,lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $query = $query . " where lms_issue.status = 'lost'";
            
            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }
        else
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.*,lms_issue.id as issue_id,lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            
            $query = ($flag == true) ? $query . " AND lms_issue.status = 'lost'" : $query . " where lms_issue.status = 'lost'";
            
            if($limit != false)
            {
                $query = $query.' '.$limit;
            }

            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }
    }

    function get_returned($filter = array(), $limit = false)
    {
        if(count($filter) <= 0 && $limit == false)
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.*,lms_issue.id as issue_id,lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $query = $query . " where lms_issue.status = 'return'";
            
            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }  
        else
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.*,lms_issue.id as issue_id,lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            
            $query = ($flag == true) ? $query . " AND lms_issue.status = 'return'" : $query . " where lms_issue.status = 'return'";
            
            if($limit != false)
            {
                $query = $query.' '.$limit;
            }

            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }  
    }

    function get_requested($filter = array(), $limit = false)
    {
        if(count($filter) <= 0 && $limit == false)
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as requested_by,lms_users.*,lms_requested.id as request_id,lms_requested.* from lms_requested inner join lms_books on lms_requested.book_id = lms_books.book_id inner join lms_users on lms_requested.user_id = lms_users.user_id';            
            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }  
        else
        {
            $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as requested_by,lms_users.*,lms_requested.id as request_id,lms_requested.* from lms_requested inner join lms_books on lms_requested.book_id = lms_books.book_id inner join lms_users on lms_requested.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_requested.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            
            if($limit != false)
            {
                $query = $query.' '.$limit;
            }

            $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
            return $data;
        }  
    }


    function get_fine_count($args = 'today') {

        if($args == 'today')
        {
            $query = 'SELECT sum(fine) as total_fine FROM lms_issue WHERE status = \'issue\' and date >= CURDATE()';
            $flag = false;
        }
        elseif($args == 'month')
        {
            $query = 'SELECT sum(fine) as total_fine FROM lms_issue WHERE status = \'issue\' and MONTH(date) = MONTH(CURDATE())';
            $flag = false;
        }
        elseif($args == 'year')
        {
            $query = 'SELECT sum(fine) as total_fine FROM lms_issue WHERE status = \'issue\' and YEAR(date) = YEAR(CURDATE())';
            $flag = false;
        }
        
        $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
        return $data[0];
    }




    function get_issued_count($args = 'today') {

        if($args == 'today')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'issue\' and date >= CURDATE()';
            $flag = false;
        }
        elseif($args == 'month')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'issue\' and MONTH(date) = MONTH(CURDATE())';
            $flag = false;
        }
        elseif($args == 'year')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'issue\' and YEAR(date) = YEAR(CURDATE())';
            $flag = false;
        }
        
        $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
        return $data;
    }




    function get_lost_count($args = 'today') {

        if($args == 'today')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'lost\' and date >= CURDATE()';
            $flag = false;
        }
        elseif($args == 'month')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'lost\' and MONTH(date) = MONTH(CURDATE())';
            $flag = false;
        }
        elseif($args == 'year')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'lost\' and YEAR(date) = YEAR(CURDATE())';
            $flag = false;
        }
        
        $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
        return $data;
    }

    function get_returned_count($args = 'today') {

        if($args == 'today')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'return\' and date >= CURDATE()';
            $flag = false;
        }
        elseif($args == 'month')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'return\' and MONTH(date) = MONTH(CURDATE())';
            $flag = false;
        }
        elseif($args == 'year')
        {
            $query = 'SELECT * FROM lms_issue WHERE status = \'return\' and YEAR(date) = YEAR(CURDATE())';
            $flag = false;
        }
        
        $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
        return $data;
    }






    function process_export($type = 'issue')
    { 
        
        if($type == 'issue')
        {
            header('Content-Type: text/csv; charset=utf-8');  
            header('Content-Disposition: attachment; filename=data.csv');

            $output = fopen("php://output", "w+");  
            fputcsv($output, array('book_id', 'book_name', 'barcode_id', 'issued_to', 'user_id', 'user_type', 'issued_date', 'fine'));  
          
            $query = 'select  lms_books.book_id, lms_books.name as book_name, lms_books.barcode_id, lms_users.name as issued_to,  lms_users.user_id as user_id,  lms_users.type as user_type, lms_issue.date as issued_date, lms_issue.fine from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            $query = ($flag == true) ? $query . " AND lms_issue.status = 'issue'" : $query . " where lms_issue.status = 'issue'";
            $smt = $this->crud->get_connection()->prepare($query);
            $smt->execute();

            while($row = $smt->fetch(PDO::FETCH_ASSOC))  
              {  
                   fputcsv($output, $row);  
              }  
              fclose($output);  
        }

        if($type == 'delayed')
        {
            header('Content-Type: text/csv; charset=utf-8');  
            header('Content-Disposition: attachment; filename=data.csv');

            $output = fopen("php://output", "w+");  
            fputcsv($output, array('book_id', 'book_name', 'barcode_id', 'issued_to', 'user_id', 'user_type', 'issued_date', 'fine'));  
          
            $query = 'select  lms_books.book_id, lms_books.name as book_name, lms_books.barcode_id, lms_users.name as issued_to,  lms_users.user_id as user_id,  lms_users.type as user_type, lms_issue.date as issued_date, lms_issue.fine from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            $query = ($flag == true) ? $query . " AND lms_issue.status = 'issue'" : $query . " where lms_issue.status = 'issue'";
            $query = $query . " AND DATE_ADD(lms_issue.date, INTERVAL lms_issue.issued_days DAY) < now()";

            $smt = $this->crud->get_connection()->prepare($query);
            $smt->execute();

            while($row = $smt->fetch(PDO::FETCH_ASSOC))  
              {  
                   fputcsv($output, $row);  
              }  
              fclose($output);  
        }

        else if($type == 'return')
        {
            header('Content-Type: text/csv; charset=utf-8');  
            header('Content-Disposition: attachment; filename=data.csv');

            $output = fopen("php://output", "w+");  
            fputcsv($output, array('book_id', 'book_name', 'barcode_id', 'issued_to', 'user_id', 'user_type', 'issued_date', 'fine'));  
          
            $query = 'select  lms_books.book_id, lms_books.name as book_name, lms_books.barcode_id, lms_users.name as issued_to,  lms_users.user_id as user_id,  lms_users.type as user_type, lms_issue.date as issued_date, lms_issue.fine from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            $query = ($flag == true) ? $query . " AND lms_issue.status = 'return'" : $query . " where lms_issue.status = 'return'";
            $smt = $this->crud->get_connection()->prepare($query);
            $smt->execute();

            while($row = $smt->fetch(PDO::FETCH_ASSOC))  
              {  
                   fputcsv($output, $row);  
              }  
              fclose($output);  
        }

        else if($type == 'lost')
        {
            header('Content-Type: text/csv; charset=utf-8');  
            header('Content-Disposition: attachment; filename=data.csv');

            $output = fopen("php://output", "w+");  
            fputcsv($output, array('book_id', 'book_name', 'barcode_id', 'issued_to', 'user_id', 'user_type', 'issued_date', 'fine'));  
          
            $query = 'select  lms_books.book_id, lms_books.name as book_name, lms_books.barcode_id, lms_users.name as issued_to,  lms_users.user_id as user_id,  lms_users.type as user_type, lms_issue.date as issued_date, lms_issue.fine from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
            $flag = false;

            if(isset($filter['from']) && isset($filter['to']))
            {
                $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
                $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

                $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
                if($flag == false) $flag = true;
            }
            $query = ($flag == true) ? $query . " AND lms_issue.status = 'lost'" : $query . " where lms_issue.status = 'lost'";
            $smt = $this->crud->get_connection()->prepare($query);
            $smt->execute();

            while($row = $smt->fetch(PDO::FETCH_ASSOC))  
              {  
                   fputcsv($output, $row);  
              }  
              fclose($output);  
        }
          
    }


    function process_notify()
    {
        $this->values = $_POST;

        if(!$this->check_form($this->values, ['subject'=>'Subject', 'message'=>'Message'])) {
            $this->flag = ERROR;
            return false;
        }


        $query = 'select lms_books.name as book_name, lms_books.*, lms_users.name as issued_to,lms_users.email as email,lms_users.*,lms_issue.id as issue_id, lms_issue.* from lms_issue inner join lms_books on lms_issue.book_id = lms_books.book_id inner join lms_users on lms_issue.user_id = lms_users.user_id';
        $flag = false;

        if(isset($filter['from']) && isset($filter['to']))
        {
            $filter['from'] =  date("Y-m-d", strtotime($filter['from']));
            $filter['to'] =  date("Y-m-d", strtotime($filter['to']));

            $query =  $query . " where lms_issue.date between '".$filter['from']."' AND '".$filter['to']."'" ;
            if($flag == false) $flag = true;
        }
        
        $query = ($flag == true) ? $query . " AND lms_issue.status = 'issue'" : $query . " where lms_issue.status = 'issue'";
        $query = $query . " AND DATE_ADD(lms_issue.date, INTERVAL lms_issue.issued_days DAY) < now()";

        $data = $this->crud->exec_query($query)->fetchAll(PDO::FETCH_CLASS);
        
        if(count($data) > 0)
        {

            foreach ($data as $key => $value) {
                if ($this->send_mail($value->email, $this->values['subject'], $this->values['message'])) 
                    {
                        // when mail has been send successfully
                        $this->flag = SUCCESS;
                        $this->message = __('Notification sent');
                        return true;
                    }
                    else {
                         
                          $this->flag = ERROR;
                          $this->message = __("Could not send Notifications");
                          return false;
                    }
            }
        }

        else
        {
            $this->flag = ERROR;
            $this->message = __("No record found");
            return false;
        }

    }




}