<?php

class booksController extends Controller {
    
    private $model;
    private $pagination_limit = 10;
    private $pagination_buttons;

    
    function __construct() { 

        $pagination_buttons = array(__('Previous'), __('Next'));

        session::check('user_id', SITE_URL.'login/');
        if(session::get('user_role') != 'A')
        {            
            session::clean_session(SITE_URL.'login/');
            die();
        }

        $this->data['footer'] = $this->model('settings')->get_settings()[0]->institute_name;
        $this->data['footer_address'] = $this->model('settings')->get_settings()[0]->address;
        $this->data['logo'] = $this->model('settings')->get_settings()[0]->logo;
            $this->data['currency'] = $this->model('settings')->get_currency()->currency;

        $this->data['categories'] =  $this->model('books')->get_categories();
    }

    function actionMain()
    {
        $modal = $this->model('books');

        $this->data['page_title'] = __('Dashboard | Library Management System');

        $this->data['total_books'] = count($modal->get_books());
        $this->data['total_issued'] = count($this->model('issue')->get_issued());
        $this->data['total_lost'] = count($this->model('issue')->get_lost());
        $this->data['total_returned'] = count($this->model('issue')->get_returned());

        $this->view('header');
        $this->view('dashboard','books');
        $this->view('footer');
    }

    function actionViewBooks($page = 1)
    {
        $modal = $this->model('books');

        if(isset($_GET['export']))
        {
            $modal->process_export();
        }
        else
        {
            $this->prg('prg_data',SITE_URL.'books/viewbooks/');
            $filter = array();
            $limit = 'limit 0,'.$this->pagination_limit;
            $this->data['pagination_link'] = '';

            if(isset($_POST['pagination']))
            {
                $page = (int)$_POST['filter'];
            }

            if($page > 1)
            {
                $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
            }

            if(isset($_POST['filter']))
            {

                if(isset($_POST['author_filter']) && $_POST['author_filter'] != '')
                {
                    $filter['author'] = $_POST['author_filter'];
                }
                if(isset($_POST['name_filter']) && $_POST['name_filter'] != '')
                {
                    $filter['name'] = $_POST['name_filter'];
                }
                if(isset($_POST['isbn_filter']) && $_POST['isbn_filter'] != '')
                {
                    $filter['isbn'] = $_POST['isbn_filter'];
                }
                if(isset($_POST['barcode_filter']) && $_POST['barcode_filter'] != '')
                {
                    $filter['barcode'] = $_POST['barcode_filter'];
                }
                if(isset($_POST['category_filter']) && $_POST['category_filter'] != '')
                {
                    $filter['category'] = $_POST['category_filter'];
                }
                $this->data['filter'] = $filter;
            }

            $this->data['books'] = $modal->get_books($filter, $limit);
            $count = count( $modal->get_books($filter));

            $this->data['modal'] = $modal;
            $this->error['error_code'] = $modal->flag;
            $this->error['message'] = $modal->message;

            if($page > 1)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
            }
            if($count > count($this->data['books']) * $page && count($this->data['books']) > 0)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
            }


            $this->data['category'] = $modal->get_categories();
            $this->data['modal'] = $modal;

            $this->data['page_title'] = __('Books | Library Management System');

            $this->view('header');
            $this->view('view_books','books');
            $this->view('footer');
        }
    }

    function actionOutStock($page = 1)
    {
        $modal = $this->model('books');
        $this->prg('prg_data',SITE_URL.'books/outstock/');

        $filter = array();
        $limit = 'limit 0,'.$this->pagination_limit;
        $this->data['pagination_link'] = '';

        if(isset($_POST['pagination']))
        {
            $page = (int)$_POST['filter'];
        }
        if($page > 1)
        {
            $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
        }

        if(isset($_POST['filter']))
        {

            if(isset($_POST['author_filter']) && $_POST['author_filter'] != '')
            {
                $filter['author'] = $_POST['author_filter'];
            }
            if(isset($_POST['name_filter']) && $_POST['name_filter'] != '')
            {
                $filter['name'] = $_POST['name_filter'];
            }
            if(isset($_POST['isbn_filter']) && $_POST['isbn_filter'] != '')
            {
                $filter['isbn'] = $_POST['isbn_filter'];
            }
            if(isset($_POST['barcode_filter']) && $_POST['barcode_filter'] != '')
            {
                $filter['barcode'] = $_POST['barcode_filter'];
            }
            if(isset($_POST['category_filter']) && $_POST['category_filter'] != '')
            {
                $filter['category'] = $_POST['category_filter'];
            }
            $this->data['filter'] = $filter;
        }

        $this->data['books'] = $modal->get_out_stock($filter, $limit);
        $count = count( $modal->get_out_stock($filter));

        $this->error['error_code'] = $modal->flag;
        $this->error['message'] = $modal->message;

        if($page > 1)
        {
            $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
        }
        if($count > count($this->data['books']) * $page && count($this->data['books']) > 0)
        {
            $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
        }


        $this->data['category'] = $modal->get_categories();

        $this->data['page_title'] = __('Books | Library Management System');

        $this->view('header');
        $this->view('view_out_stock','books');
        $this->view('footer');
    }

    function actionIssued($page = 1)
    {
        $modal = $this->model('issue');

        if(isset($_GET['export']))
        {
            $modal->process_export('issue');
        }
        else
        {
            $this->prg('prg_data',SITE_URL.'books/issued/');

            $filter = array();
            $limit = 'limit 0,'.$this->pagination_limit;
            $this->data['pagination_link'] = '';

            if(isset($_POST['pagination']))
            {
                $page = (int)$_POST['filter'];
            }
            if($page > 1)
            {
                $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
            }

            if(isset($_POST['filter']))
            {

                if(isset($_POST['from_filter']) && $_POST['from_filter'] != '')
                {
                    $filter['from'] = $_POST['from_filter'];
                }
                if(isset($_POST['to_filter']) && $_POST['to_filter'] != '')
                {
                    $filter['to'] = $_POST['to_filter'];
                }
                $this->data['filter'] = $filter;
            }

            $this->data['books'] = $modal->get_issued($filter, $limit);
            $count = count( $modal->get_issued($filter));

            if($page > 1)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
            }
            if($count > count($this->data['books']) * $page && count($this->data['books']) > 0)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
            }

            $this->data['page_title'] = __('Issued | Library Management System');

            $this->data['currency'] = $this->model('settings')->get_currency()->currency;

            $this->view('header');
            $this->view('books_issued','books');
            $this->view('footer');
        }
    }

    function actionReturned($page = 1)
    {
        $modal = $this->model('issue');

        if(isset($_GET['export']))
        {
            $modal->process_export('return');
        }
        else
            {

            $this->prg('prg_data',SITE_URL.'books/returned/');
            $filter = array();
            $limit = 'limit 0,'.$this->pagination_limit;
            $this->data['pagination_link'] = '';

            if(isset($_POST['pagination']))
            {
                $page = (int)$_POST['filter'];
            }
            if($page > 1)
            {
                $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
            }

            if(isset($_POST['filter']))
            {

                if(isset($_POST['from_filter']) && $_POST['from_filter'] != '')
                {
                    $filter['from'] = $_POST['from_filter'];
                }
                if(isset($_POST['to_filter']) && $_POST['to_filter'] != '')
                {
                    $filter['to'] = $_POST['to_filter'];
                }
                $this->data['filter'] = $filter;
            }

            $this->data['books'] = $modal->get_returned($filter, $limit);
            $count = count( $modal->get_returned($filter));

            if($page > 1)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
            }
            if($count > count($this->data['books']) * $page && count($this->data['books']) > 0)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
            }

            $this->data['page_title'] = __('Returned | Library Management System');
            
            $this->data['currency'] = $this->model('settings')->get_currency()->currency;

            $this->view('header');
            $this->view('books_returned','books');
            $this->view('footer');
        }
    }

    function actionLost($page = 1)
    {
        $modal = $this->model('issue');
        if(isset($_GET['export']))
        {
            $modal->process_export('lost');
        }
        else
        {
            $this->prg('prg_data',SITE_URL.'books/lost/');
            $filter = array();
            $limit = 'limit 0,'.$this->pagination_limit;
            $this->data['pagination_link'] = '';

            if(isset($_POST['pagination']))
            {
                $page = (int)$_POST['filter'];
            }
            if($page > 1)
            {
                $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
            }

            if(isset($_POST['filter']))
            {

                if(isset($_POST['from_filter']) && $_POST['from_filter'] != '')
                {
                    $filter['from'] = $_POST['from_filter'];
                }
                if(isset($_POST['to_filter']) && $_POST['to_filter'] != '')
                {
                    $filter['to'] = $_POST['to_filter'];
                }
                $this->data['filter'] = $filter;
            }

            $this->data['books'] = $modal->get_lost($filter, $limit);
            $count = count( $modal->get_lost($filter));

            if($page > 1)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
            }
            if($count > count($this->data['books']) * $page && count($this->data['books']) > 0)
            {
                $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
            }

            $this->data['page_title'] = __('Books Lost | Library Management System');

            $this->data['currency'] = $this->model('settings')->get_currency()->currency;

            $this->view('header');
            $this->view('books_lost','books');
            $this->view('footer');
        }
    }

    function actionAdd()
    {
        if(Helpers::isAjax())
        {
            $modal = $this->model('books');

            $modal->process_book();

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }


    function actionImport()
    {
        if(Helpers::isAjax())
        {
            $modal = $this->model('books');

            $modal->process_import();

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }

    function actionAutoAdd()
    {
            $modal = $this->model('books');

            if(isset($_POST['fetch_details'])) {
                
                $_POST['barcode_id'] = str_replace(' ', '', $_POST['barcode_id']);

                if(!is_numeric($_POST['barcode_id']))
                {
                    unset($_POST['fetch_details']);
                    $this->error['message'] = __('Invalid barcode id');
                    $this->error['error_code'] = ERROR;
                }
                else
                {

                    $book_details = $this->fetch_details($_POST['barcode_id']);

                    if(empty($book_details))
                    {
                        $this->error['message'] = __('Book not found. Please enter the details manually.');
                        $this->error['error_code'] = ERROR;
                    }

                    $this->data['title'] = isset($book_details->volumeInfo->title) ? $book_details->volumeInfo->title : '';
                    $this->data['description'] = isset($book_details->volumeInfo->description) ? $book_details->volumeInfo->description : '';
                    $authors = isset($book_details->volumeInfo->authors) ? $book_details->volumeInfo->authors : array();
                    $string = '';
                    foreach ($authors as $key => $value) {
                        if($key == count($authors) - 1)
                            $string = $string . $value;
                        else
                            $string = $string . $value . ',';
                    }

                    $this->data['authors'] = $string;

                    $this->data['publisher'] = isset($book_details->volumeInfo->publisher) ? $book_details->volumeInfo->publisher : '';
                    $this->data['publishedDate'] = isset($book_details->volumeInfo->publishedDate) ? $book_details->volumeInfo->publishedDate : '';
                    $this->data['language'] = isset($book_details->volumeInfo->language) ? $book_details->volumeInfo->language : '';
                    $identifiers = isset($book_details->volumeInfo->industryIdentifiers) ? $book_details->volumeInfo->industryIdentifiers : array() ;

                    $this->data['isbn'] = '';
                    
                    foreach ($identifiers as $key => $value) {
                        if($value->type == 'ISBN_13')
                        {
                            $this->data['isbn'] = $value->identifier;
                        }
                        elseif($value->type == 'ISBN_10')
                        {
                            $this->data['isbn'] = $value->identifier;
                        }
                        else
                        {
                            $this->data['isbn'] = $value->identifier;
                        }
                    }

                    $this->data['barcode_id'] = $_POST['barcode_id'];
                    $this->data['proceed'] = true;
                }
            }


            if(isset($_POST['submit']))
            {
                $modal->process_book();

                $this->error['message'] = $modal->message;
                $this->error['error_code'] = $modal->flag;

                if($modal->flag != SUCCESS)
                {
                    $book_details = $this->fetch_details($_POST['barcode_id']);

                    $this->data['title'] = isset($book_details->volumeInfo->title) ? $book_details->volumeInfo->title : '';
                    $this->data['description'] = isset($book_details->volumeInfo->description) ? $book_details->volumeInfo->description : '';
                    $authors = isset($book_details->volumeInfo->authors) ? $book_details->volumeInfo->authors : '';
                    $string = '';
                    foreach ($authors as $key => $value) {
                        if($key == count($authors) - 1)
                            $string = $string . $value;
                        else
                            $string = $string . $value . ',';
                    }

                    $this->data['authors'] = $string;

                    $this->data['publisher'] = isset($book_details->volumeInfo->publisher) ? $book_details->volumeInfo->publisher : '';
                    $this->data['publishedDate'] = isset($book_details->volumeInfo->publishedDate) ? $book_details->volumeInfo->publishedDate : '';
                    $this->data['language'] = isset($book_details->volumeInfo->language) ? $book_details->volumeInfo->language : '';
                    $identifiers = isset($book_details->volumeInfo->industryIdentifiers) ? $book_details->volumeInfo->industryIdentifiers : '';

                    $this->data['isbn'] = '';
                    
                    foreach ($identifiers as $key => $value) {
                        if($value->type == 'ISBN_13')
                        {
                            $this->data['isbn'] = $value->identifier;
                            break;
                        }
                        elseif($value->type == 'ISBN_10')
                        {
                            $this->data['isbn'] = $value->identifier;
                            break;
                        }
                        else
                        {
                            $this->data['isbn'] = $value->identifier;
                            break;
                        }
                    }

                    $this->data['barcode_id'] = $_POST['barcode_id'];
                    $this->data['proceed'] = true;
                }
            }

            $this->data['categories'] = $modal->get_categories();
            
           $this->data['page_title'] = __('Add Book | Library Management System');

            $this->view('header');
            $this->view('addbook','books');
            $this->view('footer');

    }
    
    private function fetch_details($isbn)
    {  
        $isbn = str_replace(' ', '', $isbn);
        if(!is_numeric($isbn))
        {
            return array();
        }

        $url = "https://www.googleapis.com/books/v1/volumes?q=isbn:" . $isbn;
        

        $book = @file_get_contents($url);
        if($book != false)
        {
            $result = json_decode($book);

              if ($result->totalItems) {
                
                // There'll be only 1 book per ISBN
                $book_details = $result->items[0];
                
                return $book_details;
              
              }

              else
              {
                return array();
              }
          }
          else
          {
            return array();
          }

    }


    function actionDeletebook()
    {
         if(Helpers::isAjax())
        {
            $modal = $this->model('books');

            if(isset($_GET['id']))
            {
                    $modal->delete_book($_GET['id']);
            }

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }

    function actionUpdatebook()
    {
        if(Helpers::isAjax())
        {
            $modal = $this->model('books');

            if(isset($_POST['type']) && $_POST['type'] == 'single')
                $modal->update_book('single');
            else
                $modal->update_book();

            $response = ($modal->flag == SUCCESS) ? '<div class="msg success"><p>'.$modal->message.'</p></div>' : '<div class="msg fail"><p>'.$modal->message.'</p></div>' ;
            $code = $modal->flag;

            echo json_encode(array('text'=>$response, 'code'=>$code));
        }
    }


    function actionEdit()
    {
        $modal = $this->model('books');
        if(isset($_GET['id']) && is_numeric($_GET['id']))
        {
            $book = $modal->check_book($_GET['id']);
            if (!empty($book) && $book != false) 
            {
                if(isset($_POST['submit']))
                {
                    if($_POST['security_token'] !== $_SESSION['token']) 
                    {
                        $this->error['message'] = __('Token Mismatch');
                        $this->error['error_code'] = ERROR;
                    }
                    else
                    {
                        $modal->update_book('all');

                        $this->error['message'] = $modal->message;
                        $this->error['error_code'] = $modal->flag;
                    }
                   
                }
        /**
         *
         *
         *
         * Generate security token to handle double form submissions
         *
         *
         *
         **/
                $this->data['token'] = md5(uniqid(rand(), TRUE).time());
                $_SESSION['token'] = $this->data['token'];

                // update form after submit
                $book = $modal->check_book($_GET['id']);

                $this->data['categories'] = $modal->get_categories();

                $this->data['title'] = $book->name;
                $this->data['description'] = $book->description;
                $this->data['language'] = $book->language;
                $this->data['category'] = $modal->get_categories($book->category);
                $this->data['publisher'] = $book->publisher;
                $this->data['photo'] = $book->photo;
                $this->data['publishedDate'] = $book->published_year;
                $this->data['authors'] = $book->author;
                $this->data['location'] = $book->location;
                $this->data['quantity'] = $book->quantity;

                $this->view('header');
                $this->view('edit_book_details','books');
                $this->view('footer');
            }
            else
            {
                header('HTTP/1.1 405'.__('Requested Method Not Allowed'));
                echo __('Invalid access! Go').'<a href="'.SITE_URL.'books/viewbooks">Home</a>';
                die();
            }
        }
        else
        {
            header('HTTP/1.1 405'.__('Requested Method Not Allowed'));
            echo __('Invalid access! Go').'<a href="'.SITE_URL.'books/viewbooks">Home</a>';
            die();
        }
    }

    function actionRequested($page = 1)
    {
        $modal = $this->model('issue');
        $this->prg('prg_data',SITE_URL.'books/requested/');

        $filter = array();
        $limit = 'limit 0,'.$this->pagination_limit;
        $this->data['pagination_link'] = '';

        if(isset($_POST['pagination']))
        {
            $page = (int)$_POST['filter'];
        }
        if($page > 1)
        {
            $limit = 'limit '.(($page * $this->pagination_limit) - $this->pagination_limit).','.($this->pagination_limit);
        }

        if(isset($_POST['filter']))
        {

            if(isset($_POST['from_filter']) && $_POST['from_filter'] != '')
            {
                $filter['from'] = $_POST['from_filter'];
            }
            if(isset($_POST['to_filter']) && $_POST['to_filter'] != '')
            {
                $filter['to'] = $_POST['to_filter'];
            }
            $this->data['filter'] = $filter;
        }

        $this->data['books'] = $modal->get_requested($filter, $limit);
        $count = count( $modal->get_requested($filter));

        if($page > 1)
        {
            $this->data['pagination_link'] .= '<button type="submit" name="filter" class="previous" value="'.($page - 1).'">Previous</button>';
        }
        if($count > count($this->data['books']) * $page && count($this->data['books']) > 0)
        {
            $this->data['pagination_link'] .= '<button type="submit" name="filter" class="next" value="'.($page + 1).'">Next</button>';
        }

        $this->data['page_title'] = __('Requested | Library Management System');
        
        $this->data['currency'] = $this->model('settings')->get_currency()->currency;


        $this->view('header');
        $this->view('books_requested','books');
        $this->view('footer');
    }

    function actionCancel($member = false, $book = false)
    {
        if (is_numeric($member) && is_numeric($book) && (!empty($member) && !empty($book))) {

                $_GET['user_id'] = $member;
                $_GET['book_id'] = $book;

            $model = $this->model('issue');
            $result = $model->cancel_requested();

            $this->error['error_code'] = $model->flag;
            $this->error['message'] = $model->message;
        }

        header("location: ".SITE_URL."books/requested");
    }
}
?>